fs=1e3; % sampling frequency
load ../signaly/x_spl  % loads signal into variable x

% generate frequency axis
N=length(x);
k=0:N-1;
f=k/N*fs;

% Spectrum - linear scale, no interpolation, no spectral leakage
% suppression
figure(1)
plot(f,abs(fft(x)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, no interpolation, spectral leakage')

% Spectrum - linear scale, interpolated, no spectral leakage suppression
figure(2)
Nfft=2000;
k=0:Nfft-1;
f=k/Nfft*fs;
plot(f,abs(fft(x,Nfft)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, interpolated, spectral leakage')

% Spectrum - linear scale, interpolated, spectral leakage suppressed with
% Hamming window
figure(3)
w=hamming(length(x));
plot(f,abs(fft(x.*w',Nfft)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, interpolated, spectral leakage suppressed (Hamming window)')

% Spectrum - logarithmic scale, interpolated, spectral leakage suppressed
% with Hamming window
figure(4)
w=hamming(length(x));
semilogy(f,abs(fft(x.*w',Nfft)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - logarithmic scale, interpolated, spectral leakage suppressed (Hamming window)')

% Spectrum - logarithmic scale, interpolated, spectral leakage suppressed
% with Blackman window
figure(5)
w=blackman(length(x));
semilogy(f,abs(fft(x.*w',Nfft)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - log scale, interpolated, spectral leakage suppressed (Blackman window)')

% Comparison of spectra computed with various weighting windows
figure(6)
X1=fft(x,Nfft);
w=hamming(length(x));
X2=fft(x.*w',Nfft);
w=blackman(length(x));
X3=fft(x.*w',Nfft);

semilogy(f,abs(X1),f,abs(X2),f,abs(X3))
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectral Leakage Comparison')
legend('no window','Hamming','Blackman')
