% Exercise 1 - generation of stochastic (random) signals

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Generation of deterministic harmonic signals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

A=1 ;
f=500 ;    % Change frequency also to  2000, 2123, 2001, 2003, 250, 50 
fs=8000 ;   
tmax=0.1 ;

% Create also a sinusoid with fs=1000 and f=250 
%   and compare with the case fs=8000 and f=2000

tt=0:1/fs:tmax ;
s1=A*sin(2*pi*f*tt) ;

figure(11)
plot(s1)
xlabel('n [samples]')
ylabel('s1[n]');

figure(12)
stem(s1)
xlabel('n [samples]')
ylabel('s1[n]');

figure(13)
plot(tt,s1,'.-')
xlabel('t [s]')
ylabel('s1(t)');
title(['Sinusoid : f=' num2str(f) 'Hz, fs=' num2str(fs) 'Hz'])

         
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Basic generation of white noise
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=1000;  % change this length also to 10000, 100000, 1000000, also to 100 or 10
         % Observe changes in histograms and values of estimated characteristics

b1=rand(1,N);

b2=randn(1,N);

figure(21);

subplot(211)
plot(b1)
title('White noise with uniform distribution')

subplot(212)
plot(b2)
title('White noise with Gaussian (normal) distribution')



% histogram computation

numclass=30; % Try also 10, 50 (and also different values of N=100, 1000, 1000000)

figure(22)
subplot(211)
hist(b1,numclass);
title('White noise with uniform distribution')
subplot(212)
hist(b2,numclass)
title('White noise with Gaussian (normal) distribution')

% Computation of mean value 
%    - observe the estimation error for particular signals (mainly related
%    to the length of analyzed vector)

mean_b1=mean(b1)
mean_b2=mean(b2)

% Variance computation - observe again estimation error

variance_b1=var(b1)
variance_b2=var(b2)

% Standard deviation computation - observe again estimation error

std_dev_b1=std(b1)
std_dev_b2=std(b2)

% Power computation (mean square value) - observe again estimation error

P_b1 =  mean(b1.^2)   % the most suitable way of the computation

% Other possibilities to compute the power of the signal
P_b1 =  sum(b1.*b1)/N ;
P_b1 =  sum(b1.^2)/length(b1) ;

b1=b1(:);         % conversion to column vector
P_b1 = b1'*b1/N ;  % power computation based on matrix multiplication 
                  % (ATTENTION. Keep proper sequenc of multiplication) 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% White noise with modified parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=1000 ;

b1=rand(1,N) ;
b10=b1-mean(b1) ;
varb10=var(b10);
b3=sqrt(0.1/varb10)*b10;

figure(23)
subplot(211);
plot(b3)
subplot(212)
hist(b3,30)
title(['b3: uniform, mean = ' num2str(mean(b3)) ', var = ' num2str(var(b3))])


b2=randn(1,N);
b4=sqrt(0.1)*b2;

figure(24)
subplot(211);
plot(b4)
subplot(212)
hist(b4,30)
title(['b4: Gaussian, mean = ' num2str(mean(b3)) ', var = ' num2str(var(b3))])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Central limit theorem (sumation of more realization of random signals)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=10000;

p=2;    % Try also for p = 2, 4, 10, 20     


% generation of 'p' realization of n. uniform distribution
nuu = rand(p,N) ;

% generation of 'p' realization of G.n.
ngg = randn(p,N) ;



figure(25)
subplot(211)
hist(nuu',30);
title(['Histograms of ' num2str(p) ' uniform noises'])

nu1=sum(nuu);   
subplot(212)
hist(nu1,30)
title(['Histogram of the sum of ' num2str(p) ' uniform noises'])

figure(26)
subplot(211)
hist(ngg',30);
title(['Histograms of ' num2str(p) ' Gaussian noises'])


ng1=sum(ngg);   
subplot(212)
hist(ng1,30)
title(['Histogram of the sum of ' num2str(p) ' Gaussian noises'])


p=1;
% generation of 'p' realization of n. uniform distribution
nuu = rand(p,N);
var_nuu=var(nuu(1,:));
% generation of 'p' realization of G.n. with the same variance
ngg = 0.1*randn(p,N);

if p>1,
    n3=sum(nuu+ngg);   
else
    n3=nuu+ngg;
end

figure(27)

subplot(211)
hist([nuu' ngg'],30);
title(['Histograms of 2 noises with different distribution'])

subplot(212)
hist(n3,30)
title(['Histogram of the sum of 2 noises'])

p=10;
% generation of 'p' realization of n. uniform distribution
nuu = rand(p,N);
var_nuu=var(nuu(1,:));
% generation of 'p' realization of G.n. with the same variance
ngg = sqrt(0.01)*randn(p,N);

if p>1,
    n3=sum(nuu+ngg);   
else
    n3=nuu+ngg;
end

figure(28)

subplot(211)
hist([nuu' ngg'],30);
title(['Histograms of 20 noises with different distribution'])

subplot(212)
hist(n3,30)
title(['Histogram of the sum of 20 noises'])


