%  cv3_dft_basic  - zero-padding in DFT computation


if exist('/etc/init.d'),
    % For linux
    addpath /home/pollak/www/html/vyu/b2m31czs/m
    addpath /home/pollak/www/html/vyu/b2m31czs/signaly
else
    % For windows
    if exist('H:\VYUKA\CZS')
        addpath H:\VYUKA\CZS\m; 
        addpath H:\VYUKA\CZS\signaly; 
    else
        addpath C:\usr\pollak\www\html\vyu\b2m31czs\m
        addpath C:\usr\pollak\www\html\vyu\b2m31czs\signaly
    end
end



%% Illustrative example for one sinusoid

A=1;
f=1231;
fs=8000;
tmax=1;

N=512;
tt=(0:N-1)./fs;

s1=sin(2*pi*f*tt);

DFTlens=[512 1024 2048 4096 8192];

for i=1:5,

    figure(i)

    NFFT=DFTlens(i);
    ff=(0:NFFT-1).*fs./NFFT ;

    S1abs=abs(fft(s1,NFFT));
    stem(ff,S1abs,'.-');
    xlabel('f [Hz]');
    ylabel('Sabs');
    title(['Sinusoid - DFT spectrum: f=' num2str(f) 'Hz, NFFT=' num2str(NFFT)])
    xlim([1000 1400])
    
end

figure(6)

NFFT=8192;
ff=(0:NFFT-1).*fs./NFFT ;
S1abs=abs(fft(s1,NFFT));
stem(ff,S1abs,'.');
xlabel('f [Hz]');
ylabel('Sabs');
title(['Sinusoid - DFT spectrum: f=' num2str(f) 'Hz, NFFT=8192 a 512'])
xlim([1000 1400])

hold on
NFFT=512;
ff=(0:NFFT-1).*fs./NFFT ;
S1abs=abs(fft(s1,NFFT));
stem(ff,S1abs,'r.');
hold off

%% Unkown signal with harmoinic components

load sig1.mat

figure(10)
subplot(211)
plot(sig1)
ylim([-2 2])
title('Sinusoids in noise')

subplot(212)
NFFT=length(sig1);
ff=(0:NFFT-1)./NFFT*fs;
stem(ff,abs(fft(sig1,NFFT)),'.');
grid
title('Spectrum for NFFT=40')

load sig2.mat

figure(11)
subplot(211)
plot(sig2)
ylim([-2 2])
title('Sinusoids in noise')

subplot(212)
NFFT=length(sig2);
ff=(0:NFFT-1)./NFFT*fs;
stem(ff,abs(fft(sig2,NFFT)),'.');
grid
title('Spectrum for NFFT=40')

% Zero-padding for measured signals, NDFT=512

NFFT=512;

figure(12)
subplot(211)
plot([sig1 zeros(1,512-length(sig1))])
xlim([0 NFFT])
title('Sinusoids in noise')

subplot(212)
ff=(0:NFFT-1)./NFFT*fs;
stem(ff,abs(fft(sig1,NFFT)),'.');
grid
title('Spectrum for NFFT=512 (zero padded)')

figure(13)
subplot(211)
plot([sig2 zeros(1,512-length(sig2))])
xlim([0 NFFT])
title('Sinusoids in noise')

subplot(212)
ff=(0:NFFT-1)./NFFT*fs;
stem(ff,abs(fft(sig2,NFFT)),'.');
grid
title('Spectrum for NFFT=512 (zero padded)')

%% mix of sinusoid (Interenet radio)

fs=1e3; % sampling frequency
load x_spl  % loads signal into variable x

% generate frequency axis
N=length(x);
k=0:N-1;
f=k/N*fs;

figure(20)
plot(x);
xlabel('n [samples]')
ylabel('x[n]|')
title('Signal waveform')

% Spectrum - linear scale, no interpolation, no spectral leakage
% suppression
figure(21)
subplot(211)
plot(f,abs(fft(x)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, no interpolation, spectral leakage')
subplot(212)
semilogy(f,abs(fft(x)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - logaritmic scale, no interpolation, spectral leakage')

% Spectrum - linear scale, interpolated, no spectral leakage suppression
figure(22)
subplot(211)
Nfft=2000;
k=0:Nfft-1;
f=k/Nfft*fs;
plot(f,abs(fft(x,Nfft)));
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, interpolated, spectral leakage')
subplot(212)
semilogy(f,abs(fft(x,Nfft))); axis tight
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - logarithmic scale, interpolated, spectral leakage')

% Spectrum - linear scale, interpolated, spectral leakage suppressed with
% Hamming window
figure(23)
w=hamming(length(x));
subplot(211)
plot(f,abs(fft(x.*w',Nfft)));  axis tight
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, interpolated, Hamming window')

% Spectrum - logarithmic scale, interpolated, spectral leakage suppressed
% with Hamming window
subplot(212)
w=hamming(length(x));
semilogy(f,abs(fft(x.*w',Nfft))); axis tight
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - logarithmic scale, interpolated, Hamming window')

% Spectrum - logarithmic scale, interpolated, spectral leakage suppressed
% with Blackman window
figure(24)
w=blackman(length(x));
subplot(211)
plot(f,abs(fft(x.*w',Nfft)));  axis tight
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - linear scale, interpolated, Blackman window')

subplot(212)
semilogy(f,abs(fft(x.*w',Nfft)));  axis tight
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectrum - logarithmic scale, interpolated, Blackman window')

% Comparison of spectra computed with various weighting windows
figure(25)
X1=fft(x,Nfft);
w=hamming(length(x));
X2=fft(x.*w',Nfft);
w=blackman(length(x));
X3=fft(x.*w',Nfft);

semilogy(f,abs(X1),f,abs(X2),f,abs(X3))
xlabel('f [Hz]')
ylabel('|X(f)|')
title('Spectral Leakage Comparison')
legend('no window','Hamming','Blackman')

