% B2M31CZS - Discrete systems of the 1st order
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% System No. 1 : y[n] = x[n] - 0.95 x[n-1] 

b1=[1 -0.95];
a1=1;

figure(1)
freqz(b1,a1,1000)
title('System No.1 : y[n] = x[n] - 0.95 x[n-1]')

figure(2)
zplane(b1,a1)
title('System No.1 : y[n] = x[n] - 0.95 x[n-1]')

figure(3)
impz(b1,a1,20)
title('System No.1 : y[n] = x[n] - 0.95 x[n-1]')

% System No. 2 : y[n] = x[n] - 0.8 y[n-1] 

b2=1;
a2=[1 0.8];

figure(4)
freqz(b2,a2,1000)
title('System No.2 : y[n] = x[n] - 0.8 y[n-1]')

figure(5)
zplane(b2, a2)
title('System No.2 : y[n] = x[n] - 0.8 y[n-1]')

figure(6)
impz(b2, a2, 100 )
title('System No.2 : y[n] = x[n] - 0.8 y[n-1]')

% System No. 3 : y[n] = x[n] + 2 y[n-1] 

b3=1;
a3=[1 -2];

figure(7)
freqz(b3,a3,1000)
title('System No.3 : y[n] = x[n] + 2 y[n-1]')

figure(8)
zplane(b3, a3)
title('System No.3 : y[n] = x[n] + 2 y[n-1]')

figure(9)
impz(b3, a3, 20 )
title('System No.3 : y[n] = x[n] + 2 y[n-1]')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 1st checked result
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sig1=loadbin('sm2.bin');     
fs=16000;

out1=filter(b1,a1,sig1);

figure(10)
subplot(211)
plot((1:length(sig1))./fs,sig1)
title('Input speech signal')
xlabel('time [s]')

subplot(212)
plot((1:length(out1))./fs,out1)
title('Output speech signal (preepmasis filter)')
xlabel('time [s]')

figure(11)

wlen=floor(0.032*fs);

subplot(211)
spectrogram(sig1,wlen,[],[],fs,'yaxis'); colormap jet; colorbar off
title('Input speech signal')

subplot(212)
spectrogram(out1,wlen,[],[],fs,'yaxis'); colormap jet; colorbar off
title('Output speech signal (preepmasis filter)')

% Freq. response is in the Figure(1)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 2nd checked result
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

sig2=randn(10000,1);     
fs=16000;

out2=filter(b2,a2,sig2);

frame_sig2=sig2(1:256);
frame_out2=out2(1:256);

figure(20)
subplot(211)
plot(frame_sig2)
title('Input signal (white noise) - 1st frame')
xlabel('Discrete time')

subplot(212)
plot(out2(1:256))
title('Output signal (colour noise) - 1st frame')
xlabel('Discrete time')


figure(21)

wlen=256;

subplot(211)
pwelch(sig2,wlen,[],[],fs,'psd'); 
title('Input signal - smoothed PSD')
ylim([-70 -20]);

subplot(212)
pwelch(out2,wlen,[],[],fs,'psd'); 
title('Output signal - smoothed PSD')
ylim([-70 -20]);

figure(22)

subplot(211)
plot(10*log10((abs(fft(frame_sig2))).^2./wlen./fs));
title('Input signal - short-time power spectrum')
ylim([-70 -20]);

subplot(212)
plot(10*log10((abs(fft(frame_out2))).^2./wlen./fs));
title('Output signal - short-time power spectrum ')
ylim([-70 -20]);

% Freq. response is in the Figure(4)






