% Filtering in the frequency domain - cyclic convolution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f=12;
fs=8000;

% Impulse response of MA filter of the length N1 (order N1-1)
N1=100;
h=ones(N1,1)./N1;

% Impulse response of the filter realizing the exponential averaging
% h=0.02*0.98.^(0:N1-1);
% h=h(:);

% Processed signal - low-frequency sine with white noise - length N2
N2=4000;  
ttt=((0:N2-1)./fs)';
x=sin(2*pi*f*ttt)+randn(N2,1) ;
x=x(:);

% Computation of linear convolution
ylin=conv(h,x) ;

figure(1)
subplot(311)
stem(h)
title('Impulse response - h[n]')
subplot(312)
plot(x)
title('Input signal - x[n]')
subplot(313)
plot(ylin)
title('Linear convolution')




% Frame processing without overlapping

NDFT=512;

slen=N2;
wlen=NDFT;

wnum=floor(slen/wlen);

ycseg=[];
for i=1:wnum,
    
    ii=(i-1)*wlen+1;
    jj=(i-1)*wlen+wlen;
   
    frame=x(ii:jj);
   
    Y=fft(h,NDFT).*fft(frame,NDFT) ;
    yframe=real(ifft(Y,NDFT));
   
    ycseg=[ycseg; yframe] ;    
    
end

figure(2)
plot(ylin)
hold on
plot(ycseg,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - no overlap, RED cyclic, BLUE linear'])


% Frame processing OLA - general window

% w=rectwin(wlen) ; 
% overlap=3*wlen/4;
% overlap=wlen/2;

w=hamming(wlen+1) ; 
w=w(1:wlen);
overlap=3*wlen/4;
% overlap=wlen/2;

% w=hanning(wlen+1) ; 
% w=w(1:wlen);
% overlap=wlen/2;

% w=blackman(wlen+1) ; 
% w=w(1:wlen);
% overlap=wlen/2;
% overlap=3*wlen/4;


wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

wnum2=floor((slen-wlen)/wstep+1);

H=fft(h,NDFT);
out=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=x(ii:jj).*w;
   
    Y=H.*fft(frame) ;
    yframe=real(ifft(Y));
   
    out(ii:jj)= out(ii:jj)+yframe./wnorm;
    
end
      

figure(3)
plot(ylin)
hold on
plot(out,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - RED OLA general, BLUE linear'])

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Band filtering

sig=loadbin('SA107S06.CS0');
fs=16000 ;

tlen=32;
wlen=floor(tlen*1e-3*fs);

% load speech_8_16_44.mat
% %%%%%%%%%%%%%%%%
% sig=sig44 ;
% fs=44100;
% wlen=1024; 
%%%%%%%%%%%%%%%
% sig=sig16 ;
% fs=16000;
% wlen=512; 
%%%%%%%%%%%%%%%
% sig=sig8 ;
% fs=8000;
% wlen=256; 

% w=rectwin(wlen) ; 
% % overlap=3*wlen/4;
% overlap=wlen/2;

w=hamming(wlen+1) ; 
w=w(1:wlen);
% overlap=3*wlen/4;
overlap=wlen/2;

% w=hanning(wlen+1) ; 
% w=w(1:wlen);
% overlap=wlen/2;

% w=blackman(wlen+1) ; 
% w=w(1:wlen);
% % overlap=wlen/2;
% overlap=3*wlen/4;

wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

slen=length(sig);
wnum2=floor((slen-wlen)/wstep+1);

f1=300;
f2=3400;
k1=round(f1/fs*wlen);
k2=round(f2/fs*wlen);
H=[zeros(k1,1); ones(k2-k1,1); zeros(wlen-2*k2+1,1);  ones(k2-k1,1); zeros(k1-1,1)];

figure(10)
ff=(0:wlen-1)./wlen.*fs;
plot(ff,H);
grid



out=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=sig(ii:jj).*w;
   
    Y=H.*fft(frame) ;
    yframe=real(ifft(Y));
   
    out(ii:jj)= out(ii:jj)+yframe./wnorm;
    
end
      

figure(11)

subplot(211)
tt=(1:length(sig))./fs;
plot(tt,sig);
xlim([0 max(tt)]);
title('Input full-band signal')

subplot(212)
tt=(1:length(out))./fs;
plot(tt,out);
xlim([0 max(tt)]);
title('filtered telephone-band signal')


figure(12)

subplot(211)
spectrogram(sig,wlen,[],[],fs,'yaxis');
title('Input full-band signal')

subplot(212)
spectrogram(out,wlen,[],[],fs,'yaxis');
title('filtered telephone-band signal')

% sound([sig; out],fs)



%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Spectral subtraction


sig2=x;

% sig=loadbin('SA107S06.CS0');
% fs=16000 ;

tlen=32;
wlen=floor(tlen*1e-3*fs);

% load speech_8_16_44.mat
% %%%%%%%%%%%%%%%%
% sig=sig44 ;
% fs=44100;
% wlen=1024; 
%%%%%%%%%%%%%%%
% sig=sig16 ;
% fs=16000;
% wlen=512; 
%%%%%%%%%%%%%%%
% sig=sig8 ;
% fs=8000;
% wlen=256; 


w=hamming(wlen+1) ; 
w=w(1:wlen);
overlap=wlen/2;

wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

slen=length(sig2);
wnum2=floor((slen-wlen)/wstep+1);

frame1=sig2(1:wlen);
Xabs=abs(fft(frame1.*w));

% Noise spectrum estimation from high frequency band
Nabs=mean(Xabs(wlen/4:wlen/2))*ones(wlen,1); 

figure(20)
ff=(0:wlen-1)./wlen.*fs;
plot(ff,20*log10(Xabs));
hold on
plot(ff,20*log10(Nabs),'LineWidth',2,'Color','r')
hold off
title('Magnitude noise spectrum in decibels')
grid

out2=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=sig2(ii:jj).*w;
   
    X=fft(frame) ;
    Xabs=abs(X);
    Xangle=angle(X);
    
    Yabs=(Xabs-Nabs).*(Xabs>Nabs) ;
    
    yframe=real(ifft(Yabs.*exp(sqrt(-1)*Xangle)));
   
    out2(ii:jj)= out2(ii:jj)+yframe./wnorm;
    
end
      

figure(21)

subplot(211)
tt=(1:length(sig2))./fs;
plot(tt,sig2);
xlim([0 max(tt)]);
title('Input noisy signal')

subplot(212)
tt=(1:length(out2))./fs;
plot(tt,out2);
xlim([0 max(tt)]);
title('Output enhanced signal')


figure(22)

subplot(211)
spectrogram(sig2,wlen,[],[],fs,'yaxis');
title('Input noisy signal')

subplot(212)
spectrogram(out2,wlen,[],[],fs,'yaxis');
title('Output denoised signal')


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Spectral subtraction for noisy speech

% This is rather clean signal - suitable for the check of distortion
sig3=loadbin('SA107S06.CS0');
fs=16000 ;

tlen=32;
wlen=floor(tlen*1e-3*fs);

load speech_8_16_44.mat
%%%%%%%%%%%%%%%%
sig3=sig16 ; % Slightly more noisy signal
fs=16000;
wlen=512; 



w=hamming(wlen+1) ; 
w=w(1:wlen);
overlap=wlen/2;

wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

slen=length(sig3);
wnum2=floor((slen-wlen)/wstep+1);

frame1=sig3(1:wlen);
Xabs=abs(fft(frame1.*w));

% Try to subtract equivalent white noise
Nabs=mean(Xabs)*ones(wlen,1);

% Correct estimation of noise spectrum from the initial non-speech part 
Nabs=sqrt(pwelch(sig3(1:fs/2),w,[],[],fs,'twosided','psd').*wlen.*fs);

figure(30)
ff=(0:wlen-1)./wlen.*fs;
plot(ff,20*log10(Xabs));
hold on
plot(ff,20*log10(Nabs),'LineWidth',2,'Color','r')
hold off
title('Magnitude noise spectrum in decibels')
grid

out3=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=sig3(ii:jj).*w;
   
    X=fft(frame) ;
    Xabs=abs(X);
    Xangle=angle(X);
    
    Yabs=(Xabs-Nabs).*(Xabs>Nabs) ;
    
    yframe=real(ifft(Yabs.*exp(sqrt(-1)*Xangle)));
   
    out3(ii:jj)= out3(ii:jj)+yframe./wnorm;
    
end
      

figure(31)

subplot(211)
tt=(1:length(sig3))./fs;
plot(tt,sig3);
xlim([0 max(tt)]);
title('Input noisy signal')

subplot(212)
tt=(1:length(out3))./fs;
plot(tt,out3);
xlim([0 max(tt)]);
title('Output enhanced signal')


figure(32)

subplot(211)
spectrogram(sig3,wlen,[],[],fs,'yaxis');
title('Input noisy signal')

subplot(212)
spectrogram(out3,wlen,[],[],fs,'yaxis');
title('Output denoised signal')

% sound([sig3; out3],fs)






