% Filtering in the frequency domain - cyclic convolution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f=12;
fs=8000;

% Impulse response of MA filter of the length N1 (order N1-1)
N1=100;
h=ones(N1,1)./N1;

% Impulse response of the filter realizing the exponential averaging
% h=0.02*0.98.^(0:N1-1);
% h=h(:);

% Processed signal - low-frequency sine with white noise - length N2
N2=4000;  
ttt=((0:N2-1)./fs)';
x=sin(2*pi*f*ttt)+randn(N2,1) ;
x=x(:);

% Computation of linear convolution
ylin=conv(h,x) ;

figure(1)
subplot(311)
stem(h)
title('Impulse response - h[n]')
subplot(312)
plot(x)
title('Input signal - x[n]')
subplot(313)
plot(ylin)
title('Linear convolution')




% Frame processing without overlapping

NDFT=512;

slen=N2;
wlen=NDFT;

wnum=floor(slen/wlen);

ycseg=[];
for i=1:wnum,
    
    ii=(i-1)*wlen+1;
    jj=(i-1)*wlen+wlen;
   
    frame=x(ii:jj);
   
    Y=fft(h,NDFT).*fft(frame,NDFT) ;
    yframe=real(ifft(Y,NDFT));
   
    ycseg=[ycseg; yframe] ;    
    
end

figure(2)
plot(ylin)
hold on
plot(ycseg,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - no overlap, RED cyclic, BLUE linear'])


% Frame processing OLA - general window

% w=rectwin(wlen) ; 
% overlap=3*wlen/4;
% overlap=wlen/2;

w=hamming(wlen+1) ; 
w=w(1:wlen);
overlap=3*wlen/4;
% overlap=wlen/2;

% w=hanning(wlen+1) ; 
% w=w(1:wlen);
% overlap=wlen/2;

% w=blackman(wlen+1) ; 
% w=w(1:wlen);
% overlap=wlen/2;
% overlap=3*wlen/4;


wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

wnum2=floor((slen-wlen)/wstep+1);

H=fft(h,NDFT);
out=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=x(ii:jj).*w;
   
    Y=H.*fft(frame) ;
    yframe=real(ifft(Y));
   
    out(ii:jj)= out(ii:jj)+yframe./wnorm;
    
end
      

figure(3)
plot(ylin)
hold on
plot(out,'r')
hold off
grid
title(['NDFT=' num2str(NDFT) ' - RED OLA general, BLUE linear'])








