%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Filtering in frequnecy domain - general OLA - Band filtering

sig=loadbin('SA107S06.CS0');
fs=16000 ;

tlen=32;
wlen=floor(tlen*1e-3*fs);

% load speech_8_16_44.mat
% %%%%%%%%%%%%%%%%
% sig=sig44 ;
% fs=44100;
% wlen=1024; 
%%%%%%%%%%%%%%%
% sig=sig16 ;
% fs=16000;
% wlen=512; 
%%%%%%%%%%%%%%%
% sig=sig8 ;
% fs=8000;
% wlen=256; 

% w=rectwin(wlen) ; 
% % overlap=3*wlen/4;
% overlap=wlen/2;

w=hamming(wlen+1) ; 
w=w(1:wlen);
% overlap=3*wlen/4;
overlap=wlen/2;

% w=hanning(wlen+1) ; 
% w=w(1:wlen);
% overlap=wlen/2;

% w=blackman(wlen+1) ; 
% w=w(1:wlen);
% % overlap=wlen/2;
% overlap=3*wlen/4;

wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

slen=length(sig);
wnum2=floor((slen-wlen)/wstep+1);

f1=300;
f2=3400;
k1=round(f1/fs*wlen);
k2=round(f2/fs*wlen);
H=[zeros(k1,1); ones(k2-k1,1); zeros(wlen-2*k2+1,1);  ones(k2-k1,1); zeros(k1-1,1)];

figure(10)
ff=(0:wlen-1)./wlen.*fs;
plot(ff,H);
grid



out=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=sig(ii:jj).*w;
   
    Y=H.*fft(frame) ;
    yframe=real(ifft(Y));
   
    out(ii:jj)= out(ii:jj)+yframe./wnorm;
    
end
      

figure(11)

subplot(211)
tt=(1:length(sig))./fs;
plot(tt,sig);
xlim([0 max(tt)]);
title('Input full-band signal')

subplot(212)
tt=(1:length(out))./fs;
plot(tt,out);
xlim([0 max(tt)]);
title('filtered telephone-band signal')


figure(12)

subplot(211)
spectrogram(sig,wlen,[],[],fs,'yaxis');
title('Input full-band signal')

subplot(212)
spectrogram(out,wlen,[],[],fs,'yaxis');
title('filtered telephone-band signal')

% sound([sig; out],fs)



%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Spectral subtraction - noisy speech with car noise


% Noisy speech
sig2=loadbin('SA107S06_auto2.CS0');
fs=16000 ;

% This is rather clean signal - suitable for the check of distortion
% sig2=loadbin('SA107S06.CS0');


tlen=32;
wlen=floor(tlen*1e-3*fs);


w=hamming(wlen+1) ; 
w=w(1:wlen);
overlap=wlen/2;

wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

slen=length(sig2);
wnum2=floor((slen-wlen)/wstep+1);

frame1=sig2(1:wlen);
Xabs=abs(fft(frame1.*w));

% Noise magnitude spectrum estimation using pwelch from the initial non-speech part 
Navg = sqrt(wlen*pwelch(sig2(1:0.5*fs),wlen,[],[],'twosided')); 

figure(20)
ff=(0:wlen-1)./wlen.*fs;
plot(ff,20*log10(Xabs));
hold on
plot(ff,20*log10(Navg),'LineWidth',2,'Color','r')
hold off
title('Magnitude noise spectrum in decibels')
grid

out2=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=sig2(ii:jj).*w;
   
    X=fft(frame) ;
    Xabs=abs(X);
    Xangle=angle(X);
    
    Yabs=(Xabs-Navg).*(Xabs>Navg) ;
    
    yframe=real(ifft(Yabs.*exp(sqrt(-1)*Xangle)));
   
    out2(ii:jj)= out2(ii:jj)+yframe./wnorm;
    
end
      

figure(21)

subplot(211)
tt=(1:length(sig2))./fs;
plot(tt,sig2);
xlim([0 max(tt)]);
title('Input noisy signal')

subplot(212)
tt=(1:length(out2))./fs;
plot(tt,out2);
xlim([0 max(tt)]);
title('Output enhanced signal')


figure(22)

subplot(211)
spectrogram(sig2,wlen,[],[],fs,'yaxis');
title('Input noisy signal')

subplot(212)
spectrogram(out2,wlen,[],[],fs,'yaxis');
title('Output denoised signal')


%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Spectral thresholding - cleaning of noisy sinusoid

% Processed signal - low-frequency sine with white noise - length N2
fs=8000 ;
f=100;
N2=4000;  
ttt=((0:N2-1)./fs)';
x=sin(2*pi*f*ttt)+randn(N2,1) ;
x=x(:);

wlen=512;


w=hamming(wlen+1) ; 
w=w(1:wlen);
overlap=wlen/2;

wstep=wlen-overlap;
wnorm=0;
for j=1:wstep,
    wnorm=wnorm+sum(w(j:wstep:wlen))/wstep;
end

slen=length(x);
wnum2=floor((slen-wlen)/wstep+1);

frame1=x(1:wlen);
Xabs=abs(fft(frame1.*w));

% Noise magnitude spectrum estimation using pwelch from the initial
% non-speech part for spectral subtraction
Navg = mean(Xabs(wlen/4:wlen/2))*ones(wlen,1); 

% Noise magnitude spectrum estimation using pwelch from the initial
% non-speech part for spectral subtraction
Xabs_mean= mean(Xabs(wlen/4:wlen/2)) ;
Xabs_std= std(Xabs(wlen/4:wlen/2)) ;
za=3 ;  %try increase/decrease the level of threshold
Nthr = Xabs_mean*ones(wlen,1)+za*Xabs_std*ones(wlen,1);

figure(30)
ff=(0:wlen-1)./wlen.*fs;
plot(ff,20*log10(Xabs));
hold on
plot(ff,20*log10(Nthr),'LineWidth',2,'Color','r','LineStyle','--')
hold off
title('Noise spectrum in decibels and chosen threshold')
grid

out2=zeros(slen,1);
for i=1:wnum2,
    
    ii=(i-1)*wstep+1;
    jj=(i-1)*wstep+wlen;
   
    frame=x(ii:jj).*w;
   
    X=fft(frame) ;
    Xabs=abs(X);
    Xangle=angle(X);
    
    % This is spectral subtraction
    % Yabs=(Xabs-Navg).*(Xabs>Navg) ;
    
    % This is spectral thresholding
    Yabs=Xabs.*(Xabs>Nthr) ;
    
    yframe=real(ifft(Yabs.*exp(sqrt(-1)*Xangle)));
   
    out2(ii:jj)= out2(ii:jj)+yframe./wnorm;
    
end
      

figure(31)

subplot(211)
tt=(1:length(x))./fs;
plot(tt,x);
xlim([0 max(tt)]);
title('Input noisy signal')

subplot(212)
tt=(1:length(out2))./fs;
plot(tt,out2);
xlim([0 max(tt)]);
title('Output enhanced signal')


figure(32)

subplot(211)
plot(abs(fft(x)));
title('Input noisy signal')

subplot(212)
plot(abs(fft(out2)));
title('Output denoised signal')







