% Filtering in the frequency domain - cyclic convolution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

f=12;
fs=8000;

% Impulse response of MA filter of the length N1 (order N1-1)
N1=100;
h=ones(N1,1)./N1;

% Impulse response of the filter realizing the exponential averaging
% h=0.02*0.98.^(0:N1-1);

% Processed signal - low-frequency sine with white noise - length N2
N2=4000;  
ttt=((0:N2-1)./fs)';
x=sin(2*pi*f*ttt)+randn(N2,1) ;

% Computation of linear convolution
ylin=conv(h,x) ;

figure(1)
subplot(311)
stem(h)
title('Impulse response - h[n]')
subplot(312)
plot(x)
title('Input signal - x[n]')
subplot(313)
plot(ylin)
title('Linear convolution')


pause


% Frame processing without overlapping

NDFT=512;

slen=N2;
wlen=NDFT;

wnum=floor(slen/wlen);

colours='gmck';

figure(3)
plot(ylin,':b')
title(['NDFT=' num2str(NDFT) ' - no overlap, RED cyclic, BLUE linear'])
hold on
a=axis;
line([a(1) a(2)],[0 0],'LineStyle',':','Color','k')
drawnow
pause(1)

ycseg=[];
for i=1:wnum,
    
    ii=(i-1)*wlen+1;
    jj=(i-1)*wlen+wlen;
   
    frame=x(ii:jj);
   
    Y=fft(h,NDFT).*fft(frame,NDFT) ;
    yframe=real(ifft(Y,NDFT));
    
    plot(ii:jj,yframe,colours(mod(i,4)+1))
    line([ii ii],[a(3) a(4)],'LineStyle',':','Color','k')
    line([jj+1 jj+1],[a(3) a(4)],'LineStyle',':','Color','k')
    drawnow
    pause(1)
   
    ycseg=[ycseg; yframe] ;    
    
end

pause

figure(3)
plot(ycseg,'r')
hold off

% drawnow
% pause(1)
% 
% % Frame processing OLS
% 
% wlen=NDFT;
% overlap=N1;
% wstep=wlen-overlap;
% slen=N2;
% 
% wnum2=floor((slen-wlen)/wstep+1);
% 
% figure(4)
% plot(ylin,':b')
% title(['NDFT=' num2str(NDFT) ' - OLS, RED cyclic, BLUE linear'])
% hold on
% a=axis;
% line([a(1) a(2)],[0 0],'LineStyle',':','Color','k')
% drawnow
% pause(1)
% 
% ycseg=[];
% for i=1:wnum2,
%     
%     ii=(i-1)*wstep+1;
%     jj=(i-1)*wstep+wlen;
%    
%     frame=x(ii:jj);
%    
%     Y=fft(h,NDFT).*fft(frame,NDFT) ;
%     yframe=real(ifft(Y,NDFT));
%    
%     if i>1,
%         ycseg=[ycseg; yframe(overlap+1:end)];
%     else
%         ycseg=[ycseg; yframe];
%     end
%       
%     plot(ii:jj,yframe,colours(mod(i,4)+1))
%     line([ii ii],[a(3) a(4)],'LineStyle',':','Color','k')
%     line([ii+overlap ii+overlap],[a(3) a(4)],'LineStyle',':','Color','k')
%     line([jj+1 jj+1],[a(3) a(4)],'LineStyle',':','Color','k')
%     drawnow
%     pause(1)
%     
% end
% 
% pause
% figure(4)
% plot(ycseg,'r')
% hold off
% drawnow
% pause(1)
% 
% % Frame processing OLA
% 
% slen=N2;
% overlap=N1-1;
% wlen=NDFT-overlap;
% 
% wnum3=floor(slen/wlen);
% 
% figure(5)
% plot(ylin,':b')
% title(['NDFT=' num2str(NDFT) ' - OLA, RED cyclic, BLUE linear'])
% hold on
% a=axis;
% line([a(1) a(2)],[0 0],'LineStyle',':','Color','k')
% drawnow
% pause(1)
% 
% 
% ycseg=zeros(slen+overlap,1);
% for i=1:wnum3,
%     
%     ii=(i-1)*wlen+1;
%     jj=(i-1)*wlen+wlen;
%    
%     frame=[x(ii:jj); zeros(overlap,1)];
%    
%     Y=fft(h,NDFT).*fft(frame,NDFT) ;
%     yframe=real(ifft(Y,NDFT));
%    
%     ycseg(ii:jj+overlap)= ycseg(ii:jj+overlap)+yframe;
% 
%     plot(ii:jj+overlap,yframe,colours(mod(i,4)+1))
%     line([ii ii],[a(3) a(4)],'LineStyle',':','Color','k')
%     line([ii+overlap ii+overlap],[a(3) a(4)],'LineStyle',':','Color','k')
%     line([jj+overlap jj+overlap],[a(3) a(4)],'LineStyle',':','Color','k')
%     line([jj+1 jj+1],[a(3) a(4)],'LineStyle',':','Color','k')
%     drawnow
%     pause(1)
%     
% end
%       
% pause
% 
% figure(5)
% plot(ycseg,'r')
% hold off
% 
% 
% 
